/*	initx.c
**
**	X window Initialization routines
**
**	Written by 	Anirudha Shimpi
**
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>

/*	program wide globals	*/

Display		*theDisplay;
int 		theScreen;
int 		theDepth;
int 		theWidth;
int		theHeight;
unsigned long	theBlackPixel;
unsigned long	theWhitePixel;

/*
	initX

	parameters : none

Fuctions performed :
	1. Open connection to X Server
	2. get default screen and depth
*/

initX()

{
theDisplay = XOpenDisplay (NULL);
if (theDisplay == NULL) {
	fprintf (stderr,"ERROR: Could not open connection to X display %s\n",
		XDisplayName (NULL));
	exit (1);
	}
theScreen = DefaultScreen (theDisplay);
theDepth = DefaultDepth (theDisplay,theScreen);
}  /* initX */

/*
	getXinfo

	parameters : none

Functions performed :
	1. give vendor/release info
	2. get defaults
	3. show defaults
*/

getXinfo ()

{

/* 	get defaults	*/

theWidth = DisplayWidth (theDisplay,theScreen);
theHeight = DisplayHeight (theDisplay,theScreen);
theBlackPixel = BlackPixel (theDisplay,theScreen);
theWhitePixel = WhitePixel (theDisplay,theScreen);

/* 	display information	*/

printf ("%s version %d of the X window system X%d R%d\n",
	ServerVendor (theDisplay),
	VendorRelease (theDisplay),
	ProtocolVersion (theDisplay),
	ProtocolRevision (theDisplay) );
if (theDepth == 1)
	printf ("Color plane depth....%d (monochrome)\n",theDepth);
else
	printf ("Color plane depth....%d\n",theDepth);
printf ("Display Width....%d\n",theWidth);
printf ("Display Height....%d\n",theHeight);
printf ("The Display %s\n",XDisplayName (theDisplay) );
} /* getXinfo */

