/*	windowx.c
**
**	window opening routines
**
**	Written by 	Anirudha Shimpi
**
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>

extern Display		*theDisplay;
extern int		theScreen;
extern int 		theDepth;
extern unsigned	long	theBlackPixel;
extern unsigned	long	theWhitePixel;

#define BORDER_WIDTH 	2

/* 
	openWindow

	parameters : x,y	location of the upper left corner
		width,height	of the window
		popupflag	>0 window is popup
				not yet implemented
		theGC		graphics context for window
		returns		window id

Functions performed :
	1. set up the window attributes
	2. open window on display
	3. tell window manager about size and location
	4. set up the GC for the window
	5. ask X to place window visibly on the screen
	6. flush out all requests to the server
*/

Window openWindow (x,y,
		width,height,
		popupflag,theGC)

int x,y,width,height,popupflag;
GC *theGC;

{
XSetWindowAttributes	theWindowAttributes;
XSizeHints		theSizeHints; 
unsigned 	long	theWindowMask;
Window			theNewWindow;

/*	set attributes	*/

theWindowAttributes.border_pixel = theBlackPixel;
theWindowAttributes.background_pixel = theWhitePixel;

theWindowAttributes.override_redirect=False;
/*	setting this to true is being unfriendly with the window manager
	asking it to leave the window alone		*/

theWindowMask = CWBackPixel | CWBorderPixel | CWOverrideRedirect;

/* 	open new window	*/

theNewWindow = XCreateWindow (theDisplay,
				RootWindow (theDisplay,theScreen),
				x,y,width,height,
				BORDER_WIDTH,
				theDepth,
				InputOutput,
				CopyFromParent,
				theWindowMask,
				&theWindowAttributes);

/*	tell the window manager	*/

theSizeHints.flags = PPosition | PSize;
theSizeHints.x = x;
theSizeHints.y = y;
theSizeHints.width = width;
theSizeHints.height=height;
XSetNormalHints (theDisplay,theNewWindow,&theSizeHints);

/*	create the GC	*/

if (createGC (theNewWindow,theGC) == 0) {
	XDestroyWindow (theDisplay,theNewWindow);
	fprintf (stderr,"Unable to create GC for window\n");
	return (0);
	}

/*	place window visibly on screen 	*/

XMapWindow (theDisplay,theNewWindow);

/*	flush	*/

XFlush (theDisplay);

return (theNewWindow);
} /* openWindow */

/*	createGC

	parameters : 
		theWindow 	Window for which the GC is to be created
		theGC 		created GC
		returns		whether successful or not
	
Functions Performed :
	1. Create a new GC for the window
	2. Set attributes of the GC
*/

createGC (theWindow,theNewGC)

Window theWindow;
GC *theNewGC;

{
XGCValues	theGCValues;

*theNewGC = XCreateGC (theDisplay,
			theWindow,
			(unsigned long) 0,
			&theGCValues);
if (*theNewGC == 0)
	return (0);
else {
	XSetForeground (theDisplay,*theNewGC,theBlackPixel);
	XSetBackground (theDisplay,*theNewGC,theWhitePixel);
	return (1);
	}
} /* createGC */
